﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace PB069.Tutorial6.BarGraphs
{
    public partial class DataForm : Form
    {
        // Vlastnosti pro preneseni informace mezi formularem a volajicim

        public int ItemValue
        {
            get { return int.Parse(valueTextBox.Text); }
            set { valueTextBox.Text = value.ToString(); }
        }

        public string ItemText
        {
            get { return nameTextBox.Text; }
            set { nameTextBox.Text = value; }
        }

        public DataForm()
        {
            InitializeComponent();
        }


        // pridany konstruktor ktery prebira vychozi hodnoty pro sva policka 
        // (napr. pri editaci jiz existujici polozky)

        // nesmime zapomenout volat puvodni konstruktor pomoci : this() nebo explicitne
        // zavolat metodu InitializeComponent abychom vytvorili ovladaci prvky formulare
        public DataForm(string itemText, int itemValue) : this()
        {
            this.ItemValue = itemValue;
            this.ItemText = itemText;
        }

        private void nameTextBox_Validating(object sender, CancelEventArgs e)
        {
            // validace textoveho pole

            // nastavenim e.Cancel na true se oznaci obsah ovladaciho prvku
            // (v tomto pripate text TextBoxu) jako spatny

            e.Cancel = string.IsNullOrWhiteSpace(nameTextBox.Text);
            if (e.Cancel)
            {
                // jestlize prvek neprosel validaci nastavime chybu na ErrorProvider,
                // aby uzivatel vedel, ze se neco deje
                errorProvider.SetError(nameTextBox, "You have to enter some name.");
            }
            else
            {
                // pokud prosel validaci, zrusime znacku o chybe nastaveni prazdne zpravy
                errorProvider.SetError(nameTextBox, "");
            }
        }

        private void valueTextBox_Validating(object sender, CancelEventArgs e)
        {
            e.Cancel = !InPositiveNumberOrZero(valueTextBox.Text);
            if (e.Cancel)
            {
                errorProvider.SetError(valueTextBox, "You have to enter positive number.");
            }
            // znacku o chybe rusime tentokrat v udalosti obsluze udalosti Validated.
        }

        private void valueTextBox_Validated(object sender, EventArgs e)
        {
            // udalost Validated se vyvola jen pokud ovladaci prvek prosel uspesne validaci
            errorProvider.SetError(valueTextBox, "");
        }

        private static bool InPositiveNumberOrZero(string p)
        {
            int i;
            return Int32.TryParse(p, out i) && i >= 0;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            // Workaround chovani modalnich formularu s validaci:

            // Volanim ValidateChildren vynutime validaci vsech 
            // ovladacich prvku na formulari a pokud vrati true je formular validni.
            if (ValidateChildren())
            {
                DialogResult = DialogResult.OK;
            }
            else
            {
                // jestlize neni formular validni, prekazime zavreni modalniho formulare
                // nastaveni vlasnosti DialogResult formulare na DialogResult.None.

                DialogResult = DialogResult.None;
                MessageBox.Show("You have entered invalid data.");
            }
        }
    }
}
