﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace PB069.Tutorial6.BarGraphs
{
    public partial class ListForm : Form
    {
        // vlasnost pro predani seznamu sloupcu volajicimu formulari
        public Column[] Columns
        {
            get
            {
                // prevedeme polozky seznamu (ListBoxu) na pole sloupcu
                // tento prevod je v poradku, protoze do ListBoxu ukladame jen sloupce,
                // takze mame jistotu, ze se cely prevod zdari

                Column[] columns = new Column[dataListBox.Items.Count];
                for (int index = 0; index < dataListBox.Items.Count; index++)
                {
                    columns[index] = (Column)dataListBox.Items[index];
                }
                return columns;

                // kratsi varianta tohoto getteru pomoci LINQ:
                // return dataListBox.Items.Cast<Column>().ToArray();
            }
        }

        public ListForm()
        {
            InitializeComponent();
        }

        // pridany konstruktor formulare prebirajici navic jako parametr seznam 
        // sloupecku v grafu, aby je mohl zobrazit v seznamu.
        public ListForm(Column[] columns)
        {
            InitializeComponent();
            foreach (var column in columns)
            {
                dataListBox.Items.Add(column);
            }
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            // modalne zobrazime datovy formular pro zadani informaci o novem sloupci
            DataForm frm = new DataForm();
            if (frm.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                // pokud byl formular potvrzen tlacitkem OK pomoci nami pridanych vlastnosti
                // zjistime nove udaje a vytvorime sloupec ktery pridame do seznamu
                Column column = new Column
                               {
                                   Value = frm.ItemValue,
                                   Text = frm.ItemText
                               };
                dataListBox.Items.Add(column);
            }
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            // jestlize je vybrany nejaky radek seznamu
            if (dataListBox.SelectedItems.Count > 0)
            {
                // pro jistotu se jeste zeptame uzivatele, jestli chce 
                // sloupecek opravdu odstranit
                DialogResult result = MessageBox.Show("Do you really want to remove bar from the graph?",
                                "Remove Bar",
                                MessageBoxButtons.YesNo,
                                MessageBoxIcon.Question);
                if (result == System.Windows.Forms.DialogResult.Yes)
                {
                    dataListBox.Items.Remove(dataListBox.SelectedItems[0]);
                }
            }
        }

        private void changeButton_Click(object sender, EventArgs e)
        {
            // jestlize je vybrany nejaky radek seznamu...
            if (dataListBox.SelectedItems.Count > 0)
            {
                // vybrany radek v seznamu prevedeme na sloupecek reprezentovany
                // tridou Column; musime pretypovat protoze prvky seznamu jsou typu Object
                Column column = (Column)dataListBox.SelectedItem;

                // vytvorime a zobrazime dialog pro zadani hodnot
                // tomuto dialogu jsme napsali i druhy konstruktor, kterym se 
                // predavaji informace, ktere ma zobrazit
                DataForm frm = new DataForm(column.Text, column.Value);

                if (frm.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    // po potvrzeni dialogu tlacitkem OK zmenime informace
                    // ve vybranem sloupci
                    column.Value = frm.ItemValue;
                    column.Text = frm.ItemText;

                    // protoze ovladaci prvek ListBox si pro lepsi vykon cachuje polozky
                    // a metoda, ktera by nam umoznila je obnovit je bohuzel protected

                    // pouzijeme maly trik: zmeny cele polozky si ListBox uz vsimne, takze 
                    // 'zmenime' polozku seznamu na sebe samu.
                    int index = dataListBox.Items.IndexOf(column);
                    dataListBox.Items[index] = dataListBox.Items[index];
                }
            }
        }
    }
}
