﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace PB069.Tutorial6.BarGraphs
{
    public partial class MainForm : Form
    {
        // data pro vykreslování grafu 
        // NOTE: opravdu tu ma byt column a ne bar, protoze jsem tim puvodne 
        // myslel sloupec tabulky, ktery se vykresloval do ruznych grafu
        private Column[] columns = new Column[0];

        // stetec pro vykresleni pozadi 
        // - uschovany at ho nemusime v OnPaint po kazde vytvaret znovu
        private Brush backgroundBrush;

        // font pro vypis popisku grafu
        // - uschovany at ho nemusime po kazde vytvaret znovu
        private Font barLabelFont;

        public MainForm()
        {
            InitializeComponent();

            // aktualizujeme stetec pro vykresleni
            UpdateBackgroundBrush();

            barLabelFont = new Font("Arial", 12.0f, FontStyle.Italic);
        }

        private void graphDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateData();
        }

        private void UpdateData()
        {
            // zobrazime formular a predame mu aktualni data, aby je mohl 
            // vypsat v seznamu
            ListForm frm = new ListForm(columns);
            if (frm.ShowDialog() == DialogResult.OK)
            {
                // po zavreni dialogu z nej preneseme data zpatky
                columns = frm.Columns;

                // vynutime prekresleni grafu
                mainToolStripContainer.ContentPanel.Invalidate();
            }
        }

        private void RenderGraph(Graphics graphics)
        {
            // metoda pro vykresleni grafu


            // jestlize neni co vykreslovat, tak muzeme hned skoncit
            if (columns.Length == 0)
                return;

            // vykreslime cernou caru pres pod grafem:
            //   pres celou sirku panelu, 20px od spodniho okraje
            graphics.DrawLine(
                Pens.Black,
                0,
                mainToolStripContainer.ContentPanel.Height - 20,
                mainToolStripContainer.ContentPanel.Width,
                mainToolStripContainer.ContentPanel.Height - 20
            );

            // jednotlive sloupce grafu budeme vykreslovat tak, aby vyplnili 
            // celou dostupnou sirku a 10 pixelu od leveho okraje a 10 volnymi pixely mezi sloupci

            int columnWidth = (mainToolStripContainer.ContentPanel.Width-10) / (columns.Length) - 10;

            // chceme aby sloupec s nejvetsi hodnotou byl nejvyssi a pak vyska
            // ostatnich sloupcu pak pomerove podle nej

            // musime si tedy zjistit nejvetsi hodnotu ze vsech sloupcu
            // to muzeme udelat nekolika zpusoby, naprikald prochazenim v cyklu 
            // a hledanim maxima (klasicka programatorska uloha) nebo pouzitim 
            // LINQ:
            double maxValue = this.columns.Max(c => c.Value);

            // spocitam si dostupnou vysku pro vykresleni grafu, s tim, ze chci 20 volnych 
            // pixelu pod grafem a 40 nad grafem
            int maxHeight = this.mainToolStripContainer.ContentPanel.Height - 60;

            // ukazatel na pozici na ose X, kde jsem kreslil naposledy
            int lastPositionFromLeft = 10;

            // vytvorim tridu urcujici formatovani textu
            StringFormat format = new StringFormat();
            format.Alignment = StringAlignment.Center; // zarovnani na stred

            // a nyni vykreslim postupne vsechny sloupecky grafu
            foreach (Column column in columns)
            {
                // prepoctu hodnotu sloupce na jeho vysku v pixelech vzhledem k nejvetsi hodnote
                int height = (int)(maxHeight * (column.Value / maxValue));

                // vykreslim obdelniky pro sloupecek - nejprve pozadi a pak ramecek
                graphics.FillRectangle(Brushes.Red, lastPositionFromLeft, 40 + (maxHeight - height), columnWidth, height);
                graphics.DrawRectangle(Pens.Black, lastPositionFromLeft, 40 + (maxHeight - height), columnWidth, height);

                // vypisu text do obdelniku v hornich 34 pixelech nad grafem
                graphics.DrawString(column.ToString(), 
                    SystemFonts.DefaultFont, 
                    Brushes.Black, 
                    new Rectangle(lastPositionFromLeft, 3, columnWidth, 37), 
                    format);

                // aktualizujeme pozici posledniho vykresleni o sirku sloupce a 10 px mezeru
                lastPositionFromLeft += columnWidth + 10;
            }
        }

        private void RenderBackground(Graphics graphics)
        {
            graphics.Clear(Color.White);

            if (backgroundBrush != null)
            { 
                graphics.FillRectangle(backgroundBrush, mainToolStripContainer.ContentPanel.ClientRectangle);
            }
        }

        private void UpdateBackgroundBrush()
        {
            backgroundBrush = new LinearGradientBrush(
                mainToolStripContainer.ClientRectangle, 
                Color.WhiteSmoke, 
                Color.Gray, 
                LinearGradientMode.ForwardDiagonal);
        }

        private void mainToolStripContainer_ContentPanel_Paint(object sender, PaintEventArgs e)
        {
            // vykresleni grafu na panel

            RenderBackground(e.Graphics);
            RenderGraph(e.Graphics);
        }

        private void mainToolStripContainer_ContentPanel_Resize(object sender, EventArgs e)
        {
            // pokud se zmeni velikost panelu je potreba upravit i velikost 
            // stetce s barevnym prechodem, protoze pri jeho vytvoreni se zada
            // oblast na ktere vytvari prechod a pak se zacina opakovat.

            UpdateBackgroundBrush();
        }

        private void updateDataToolStripButton_Click(object sender, EventArgs e)
        {
            UpdateData();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox abox = new AboutBox();
            abox.ShowDialog();
        }
    }
}
